AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

local SpellPowerMultiplier = {
	1459--[[Arcane Brilliance]],
	77747--[[Burning Wrath]],
	109773--[[Dark Intent]],
}

local MagicVulnerability = {
	1490--[[Curse of the Elements]],
	58410--[[Master Poisoner]],
	116202--[[Aura of the Elements]],
}

a.spells["Auto Attack"] = {
	ID = 6603--[[Auto Attack]],
	EnemyTargetNeeded = true,
	Melee = 1,
	NotIfActive = 1,
	Check = function(z)
		z.FlashColor = s.If(x.ActiveEnemy, "White", "Pink")
		return x.NoCC
	end,
}

a.spells["Shoot"] = {
	ID = 3018--[[Shoot]],
	EnemyTargetNeeded = true,
	NotIfActive = 1,
	Check = function(z)
		z.FlashColor = s.If(x.ActiveEnemy, "White", "Pink")
		return x.NoCC and not s.Shooting()
	end,
}

a.spells["Pet Attack"] = {
	EnemyTargetNeeded = true,
	Override = function(z)
		if x.NoCC and x.PetAlive and x.PetActiveEnemy and not s.SameTargetAsPet() then
			s.FlashPet(PET_ACTION_ATTACK)
		end
	end,
	FlashColor = "Pink",
	NoGCD = true,
}

a.spells["Pet Assist"] = {
	Override = function(z)
		if x.ActiveEnemy and x.PetAlive and not x.PetActiveEnemy then
			s.FlashPet(PET_MODE_ASSIST)
		end
	end,
	FlashColor = "Pink",
	NoGCD = true,
}

a.spells["Dark Intent"] = {
	ID = 109773--[[Dark Intent]],
	Buff = SpellPowerMultiplier,
	EarlyRefresh = 10,
	NoRangeCheck = 1,
	BuffUnit = "raid",
	FlashColor = "Green",
}

a.spells["Curse of the Elements"] = {
	ID = 1490--[[Curse of the Elements]],
	Debuff = MagicVulnerability,
	EarlyRefresh = 5,
	EnemyTargetNeeded = true,
	Check = function(z)
		return x.NoCC
	end,
}

a.spells["Summon Felhunter ooc"] = {
	ID = 691--[[Summon Felhunter]],
	Check = function(z)
		return not x.PetAlive
	end,
	FlashID = {691--[[Summon Felhunter]], 112869--[[Summon Observer]], 9--[[Summon Demon]]}
}

a.spells["Summon Felguard ooc"] = {
	ID = 30146--[[Summon Felguard]],
	Check = function(z)
		return not x.PetAlive
	end,
	FlashID = {30146--[[Summon Felguard]], 112870--[[Summon Wrathguard]], 9--[[Summon Demon]]}
}

a.spells["Dark Soul: Misery"] = {
	ID = 113860--[[Dark Soul: Misery]],
	Check = function(z)
		return s.InCombat()
	end,
	NoGCD = true,
	Continue = true,
}

a.spells["Dark Soul: Knowledge"] = {
	ID = 113861--[[Dark Soul: Knowledge]],
	Check = function(z)
		return s.InCombat() and s.Power("player", SPELL_POWER_DEMONIC_FURY) >= 500
	end,
	NoGCD = true,
	Continue = true,
}

a.spells["Dark Soul: Instability"] = {
	ID = 113858--[[Dark Soul: Instability]],
	Check = function(z)
		return x.NoCC
	end,
	NoGCD = true,
	Continue = true,
}

a.spells["Grimoire: Felhunter"] = {
	ID = 108501--[[Grimoire of Service]],
	Check = function(z)
		return s.InCombat() and s.HasTalent(108501--[[Grimoire of Service]])
	end,
	FlashColor = "Yellow",
	FlashID = {111897--[[Grimoire: Felhunter]], 108501--[[Grimoire of Service]]},
	NoGCD = true,
	Continue = true,
}

a.spells["Grimoire: Felguard"] = {
	ID = 108501--[[Grimoire of Service]],
	Check = function(z)
		return s.InCombat() and s.HasTalent(108501--[[Grimoire of Service]])
	end,
	FlashColor = "Yellow",
	FlashID = {111898--[[Grimoire: Felguard]], 108501--[[Grimoire of Service]]},
	NoGCD = true,
	Continue = true,
}

a.spells["Grimoire of Sacrifice"] = {
	ID = 108503--[[Grimoire of Sacrifice]],
	Check = function(z)
		return x.PetAlive and s.InCombat() and s.HasTalent(108503--[[Grimoire of Sacrifice]])
	end,
	FlashColor = "Yellow",
	NoGCD = true,
	Continue = true,
}

a.spells["Summon Felhunter (Sacrifice down)"] = {
	ID = 691--[[Summon Felhunter]],
	Check = function(z)
		return s.InCombat() and s.HasTalent(108503--[[Grimoire of Sacrifice]])  and not s.Buff(108503--[[Grimoire of Sacrifice]], "player") and not x.PetAlive
	end,
	FlashID = {691--[[Summon Felhunter]], 112869--[[Summon Observer]]}
}

a.spells["Summon Felguard (Sacrifice down)"] = {
	ID = 30146--[[Summon Felguard]],
	Check = function(z)
		return s.InCombat() and s.HasTalent(108503--[[Grimoire of Sacrifice]])  and not s.Buff(108503--[[Grimoire of Sacrifice]], "player") and not x.PetAlive
	end,
	FlashID = {30146--[[Summon Felguard]], 112870--[[Summon Wrathguard]]}
}

a.spells["Summon Doomguard"] = {
	ID = 18540--[[Summon Doomguard]],
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
	Continue = true,
}

a.spells["Summon Terrorguard"] = {
	ID = 112927--[[Summon Terrorguard]],
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
	Continue = true,
}

a.spells["Soulburn at start & end of Dark Soul"] = {
	ID = 74434--[[Soulburn]],
	Buff = 74434--[[Soulburn]],
	BuffUnit = "player",
	RequireMyBuff = 113860--[[Dark Soul: Misery]],
	RequireBuffUnit = "player",
	Check = function(z)
		duration = s.MyBuffDuration(113860--[[Dark Soul: Misery]], "player")
		return duration > 0 and (duration >= 18.5 or duration <= 1.5)
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Soul Swap with Soulburn"] = {
	ID = 86121--[[Soul Swap]],
	RequireMyBuff = 74434--[[Soulburn]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Haunt"] = {
	ID = 48181--[[Haunt]],
	Debuff = 48181--[[Haunt]],
	RequireMyDebuff1 = 980--[[Agony]],
	RequireMyDebuff2 = 172--[[Corruption]],
	RequireMyDebuff3 = 30108--[[Unstable Affliction]],
	EarlyRefresh = 1,
	Check = function(z)
		return x.NoCC and not s.SpellOrAuraDelay(48181--[[Haunt]], "target", true) and s.Power("player", SPELL_POWER_SOUL_SHARDS ) > 1
	end,
	EnemyTargetNeeded = true,
}

a.spells["Agony refresh early if not channeling"] = {
	ID = 980--[[Agony]],
	MyDebuff = 980--[[Agony]],
	EarlyRefresh = 12,
	CheckLast = function(z)
		local ttd = s.TimeToDie("target") or 8
		return x.NoCC and ttd >= 8
	end,
	NoStopChannel = true,
	EnemyTargetNeeded = true,
}

a.spells["Agony"] = {
	ID = 980--[[Agony]],
	MyDebuff = 980--[[Agony]],
	EarlyRefresh = 2,
	Check = function(z)
		local ttd = s.TimeToDie("target") or 8
		return x.NoCC and ttd >= 8
	end,
	EnemyTargetNeeded = true,
}

a.spells["Corruption refresh early if not channeling"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	EarlyRefresh = 9,
	CheckLast = function(z)
		local ttd = s.TimeToDie("target") or 0
		local cl = s.MyDebuffDuration(172--[[Corruption]], "target") or 0
		if ttd < cl then
			return false
		end
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
	NoStopChannel = true,
}

a.spells["Corruption"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	EarlyRefresh = 2,
	Check = function(z)
		local ttd = s.TimeToDie("target") or 0
		local cl = s.MyDebuffDuration(172--[[Corruption]], "target") or 0
		if ttd < cl then
			return false
		end
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Unstable Affliction refresh early if not channeling"] = {
	ID = 30108--[[Unstable Affliction]],
	MyDebuff = 30108--[[Unstable Affliction]],
	EarlyRefresh = 7,
	CheckLast = function(z)
		local ttd = s.TimeToDie("target") or 5
		return x.NoCC and ttd >= 5
	end,
	EnemyTargetNeeded = true,
	NoStopChannel = true,
}

a.spells["Unstable Affliction"] = {
	ID = 30108--[[Unstable Affliction]],
	MyDebuff = 30108--[[Unstable Affliction]],
	EarlyRefresh = 2,
	Check = function(z)
		local ttd = s.TimeToDie("target") or 5
		return x.NoCC and ttd >= 5
	end,
	EnemyTargetNeeded = true,
}

a.spells["Doom"] = {
	ID = 124913--[[Metamorphosis: Doom]],
	MyDebuff = 603--[[Doom]],
	EarlyRefresh = 30,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Shadowburn"] = {
	ID = 17877--[[Shadowburn]],
	Check = function(z)
		return x.NoCC
	end,
	FlashColor = "Orange",
	EnemyTargetNeeded = true,
}

a.spells["Chaos Bolt"] = {
	ID = 116858--[[Chaos Bolt]],
	Check = function(z)
		return x.NoCC and s.BuffStack(117828--[[Backdraft]]) < 3
	end,
	EnemyTargetNeeded = true,
}

a.spells["Conflagrate"] = {
	ID = 17962--[[Conflagrate]],
	MyBuff = 117828--[[Backdraft]],
	BuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Immolate"] = {
	ID = 348--[[Immolate]],
	MyDebuff = 348--[[Immolate]],
	EarlyRefresh = 7,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Incinerate"] = {
	ID = 29722--[[Incinerate]],
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Drain Soul as finisher"] = {
	ID = 1120--[[Drain Soul]],
	Check = function(z)
		return x.NoCC and not s.NotDieing("target")
	end,
	EnemyTargetNeeded = true,
}

a.spells["Life Tap if mana < 35%"] = {
	ID = 1454--[[Life Tap]],
	Check = function(z)
		return s.InCombat() and s.MaxPower("player", SPELL_POWER_MANA) > 0 and s.PowerPercent("player", SPELL_POWER_MANA) < 35
	end,
}

a.spells["Life Tap while moving if mana < 80%"] = {
	ID = 1454--[[Life Tap]],
	Check = function(z)
		return s.Moving("player") and s.InCombat() and s.PowerPercent("player", SPELL_POWER_MANA) < 80
	end,
}

a.spells["Life Tap Demo"] = {
	ID = 1454--[[Life Tap]],
	Check = function(z)
		return s.InCombat() and not s.Buff(103958--[[Metamorphosis]], "player") and s.PowerPercent("player", SPELL_POWER_MANA) < 50
	end,
}

a.spells["Malefic Grasp"] = {
	ID = 103103--[[Malefic Grasp]],
	Check = function(z)
		return x.NoCC
	end,
	NoStopChannel = true,
	EnemyTargetNeeded = true,
}

a.spells["Fel Flame while moving"] = {
	ID = 77799--[[Fel Flame]],
	Check = function(z)
		return s.Moving("player") and s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Life Tap"] = {
	ID = 1454--[[Life Tap]],
	Check = function(z)
		return s.InCombat()
	end,
	NoStopChannel = true,
}

a.spells["Command Demon - Wrathstorm"] = {
	ID = 119915--[[Wrathstorm]],
	Override = function(z)
		return true
	end,
	NoGCD = true,
}

a.spells["Command Demon - Felstorm"] = {
	ID = 119914--[[Felstorm]],
	Override = function(z)
		return true
	end,
	NoGCD = true,
}

a.spells["Wrathstorm"] = {
	Type = "pet",
	ID = 115831--[[Wrathstorm]],
	Check = function(z)
		return s.InCombat()
	end,
	RunLast = function(z)
		a.Flash("Command Demon - Wrathstorm")
	end,
	NoGCD = true,
}

a.spells["Felstorm"] = {
	Type = "pet",
	ID = 89751--[[Felstorm]],
	Check = function(z)
		return s.InCombat()
	end,
	RunLast = function(z)
		a.Flash("Command Demon - Felstorm")
	end,
	NoGCD = true,
}

a.spells["Metamorphosis"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	NotIfActive = true,
	FlashBlink = true,
	Check = function(z)
		local ttd = s.TimeToDie("target") or 0
		return x.NoCC and ((s.MyBuffDuration(113861--[[Dark Soul: Knowledge]], "player") > 0) or (s.MyDebuffDuration(172--[[Corruption]], "target") > 2 and s.MyDebuffDuration(172--[[Corruption]], "target") < 5) or (s.Power("player", SPELL_POWER_DEMONIC_FURY) >= 900) or (s.Power("player", SPELL_POWER_DEMONIC_FURY) >= ttd*30))
	end,
	NoGCD = true,
	EnemyTargetNeeded = true,
}

a.spells["Cancel Metamorphosis"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	FlashBlink = true,
	Override = function(z)
		local ttd = s.TimeToDie("target") or 0
		return s.InCombat() and s.Buff(103958--[[Metamorphosis]], "player") and ((s.MyDebuffDuration(172--[[Corruption]], "target") > 20) or (s.MyDebuffDuration(172--[[Corruption]], "target") == 0)) and s.MyBuffDuration(113861--[[Dark Soul: Knowledge]], "player") == 0 and s.Power("player", SPELL_POWER_DEMONIC_FURY) <= 750 and ttd > 30
	end,
	FlashColor = "Pink",
	NoGCD = true,
	Continue = true,
}

a.spells["Imp Swarm"] = {
	ID = 104316--[[Imp Swarm]],
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Hand of Gul'dan"] = {
	ID = 105174--[[Hand of Gul'dan]],
	MyDebuff = 47960--[[Shadowflame]],
	Check = function(z)
		return x.NoCC and not s.SpellOrAuraDelay(105174--[[Hand of Gul'dan]], "target", any)
	end,
	EnemyTargetNeeded = true,
}

a.spells["Soul Fire"] = {
	ID = 6353--[[Soul Fire]],
	RequireMyBuff = 122355--[[Molten Core]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC and (s.MyBuffStack(103958--[[Metamorphosis]], "player") == 0 or not s.NotDieing("target"))
	end,
	EnemyTargetNeeded = true,
}

a.spells["Doom"] = {
	ID = 124913--[[Metamorphosis: Doom]],
	MyDebuff = 603--[[Doom]],
	EarlyRefresh = 30,
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	FlashID = {124913--[[Metamorphosis: Doom]], 603--[[Doom]], 172--[[Corruption]]},
	EnemyTargetNeeded = true,
}

a.spells["Touch of Chaos"] = {
	ID = 112089--[[Metamorphosis: Touch of Chaos]],
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Shadow Bolt"] = {
	ID = 686--[[Shadow Bolt]],
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}